@@ -758,9 +758,15 @@
             (progn
               (insert "  extern id java_swarmEnvironmentCreating;\n")
               (insert "  extern void swarm_java_constructors ();\n")
+              (insert "  extern void __objc_exec_class_for_all_initial_modules ();\n")
+              (insert "  extern void swarm_force_references ();\n")
+              (insert "  extern void swarm_java_predispatch ();\n")
               (insert "  extern void swarm_directory_associate_objects_startup (jobject swarmEnvironment);\n")
               (insert "  jobject nextPhase;\n\n")
               (insert "  jniEnv = env;\n")
+	      (insert "  __objc_exec_class_for_all_initial_modules ();\n")
+	      (insert "  swarm_force_references ();\n")
+              (insert "  swarm_java_predispatch ();\n")
               (insert "  swarm_java_constructors ();\n")
               (insert "  java_create_refs ();\n")
               (insert "  defobj_init_java_call_tables ((void *) env);\n")
