@@ -1,7 +1,21 @@
 // Swarm library. Copyright  1996-2000 Swarm Development Group.
-// This library is distributed without any warranty; without even the
-// implied warranty of merchantability or fitness for a particular purpose.
-// See file LICENSE for details and terms of copying.
+// This program is free software; you can redistribute it and/or modify
+// it under the terms of the GNU General Public License as published by
+// the Free Software Foundation; either version 2 of the License, or
+// (at your option) any later version.
+//
+// This program is distributed in the hope that it will be useful, but
+// WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+// General Public License for more details.
+//
+// You should have received a copy of the GNU General Public License
+// along with this program; if not, write to the Free Software
+// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
+// USA
+// 
+// The Swarm Development Group can be reached via our website at:
+// http://www.swarm.org/
 
 /*
 Name:         Schedule.m
@@ -12,8 +26,16 @@ Library:      activity
 #import <activity/Schedule.h>
 #import <activity/ActionGroup.h>
 #import <defobj/defalloc.h>
+#import <activity/classes.h>
 #include <misc.h> // abort
 
+#import <collections/List_mlinks.h> // beginMlinksList
+#import <defobj/macros.h>
+#import <collections/macros.h>
+#import <activity/macros.h>
+
+#define FAST
+
 @implementation Schedule_c
 
 PHASE(Creating)
@@ -176,8 +198,8 @@ PHASE(Using)
 
   // create merge action for use by swarm in merging schedule subactivities
 
-  newIndex = newActivity->currentIndex;
-  mergeAction = [swarmZone allocIVarsComponent: id_ActionMerge_c];
+  newIndex = (ScheduleIndex_c *)newActivity->currentIndex;
+  mergeAction = ALLOCIVARSCOMPONENT (swarmZone, id_ActionMerge_c);
   setMappedAlloc (mergeAction);
   mergeAction->subactivity = newActivity;
   mergeAction->collectionOfActions = self;
@@ -185,7 +207,7 @@ PHASE(Using)
 
   // set the starting and current times of the new activity
 
-  swarmIndex = swarmActivity->currentIndex;
+  swarmIndex = (ScheduleIndex_c *)swarmActivity->currentIndex;
   newIndex->currentTime = newIndex->startTime = swarmIndex->currentTime;
 
   // Advance the new index to its first action. The nextAction: message
@@ -208,11 +230,19 @@ createGroup (Schedule_c *self)
   // create new concurrent group to receive new action
 
   newGroup = [self->concurrentGroupType create: getCZone (zone)];
-  setBit (((Collection_any *) newGroup)->bits, BitConcurrentGroup, 1);
-  setBit (((Collection_any *) newGroup)->bits, BitAutoDrop,
-          getBit( self->bits, BitAutoDrop));
 
-  newAction = [zone allocIVarsComponent: id_ActionConcurrent_c];
+  // forced on
+  setBit (((Collection_any *) newGroup)->bits, BitConcurrentGroup, 1);
+
+  // forced to policy of Schedule
+  setBit (((Collection_any *) newGroup)->bits, BitAutoDrop,
+          getBit (self->bits, BitAutoDrop));
+
+  // if either sets these flags, use it
+  ((Collection_any *) newGroup)->bits |= getBit (self->bits, BitRandomized);
+  ((Collection_any *) newGroup)->bits |= getBit (self->bits, BitConcurrent);
+
+  newAction = ALLOCIVARSCOMPONENT (zone, id_ActionConcurrent_c);
   setMappedAlloc (newAction);
   newAction->owner = (id) self;
   newAction->concurrentGroup = (id) newGroup;
@@ -237,10 +267,10 @@ _update_mergeSchedules (Schedule_c *self,
   ScheduleIndex_c *mergeScheduleIndex;
   ActionMerge_c *mergeAction; 
   Schedule_c *mergeSchedule;
-  
+
   if (!mergeScheduleActivity)
     return;
-  mergeScheduleIndex = mergeScheduleActivity->currentIndex;
+  mergeScheduleIndex = (ScheduleIndex_c *)mergeScheduleActivity->currentIndex;
   mergeSchedule = (Schedule_c *) mergeScheduleIndex->collection;
   if (mergeScheduleIndex->currentTime > tVal)
     mergeScheduleIndex->currentTime = tVal;
@@ -263,9 +293,9 @@ _update_mergeSchedules (Schedule_c *self,
 	      aMember = [index next];
 	      if (!aMember) 
 		raiseEvent (InvalidOperation, 
-			    "> MergeSchedule is invalid. There is no",
-			    "> mergeAction for schedule where action",
-			    "> should be inserted!");
+			    "> MergeSchedule is invalid. There is no\n"
+			    "> mergeAction for schedule where action\n"
+			    "> should be inserted!\n");
 	      if (((ActionMerge_c *) aMember)->collectionOfActions == self)
 		{
 		  mergeAction = aMember;
@@ -273,7 +303,7 @@ _update_mergeSchedules (Schedule_c *self,
 		  break;
 		}
 	    }
-          [index drop];
+          DROP (index);
 	} 
       else 
         mergeAction = [mergeSchedule removeKey: (id) oldTime];     
@@ -284,13 +314,13 @@ _update_mergeSchedules (Schedule_c *self,
 static void
 ensureLeadingMerge (Schedule_c *self, id <Index> index, timeval_t tVal)
 {
-  if (![index prev])
+  if (!MAP_INDEX_PREV (index))
     {
       id indexrefs;
       ScheduleActivity_c *activity;
 
-      indexrefs = [self->activityRefs begin: scratchZone];
-      [indexrefs setLoc: Start];
+      indexrefs = MLIST_BEGIN_ZONE (self->activityRefs, scratchZone);
+      MLIST_INDEX_SETLOC (indexrefs, Start);
       activity = [indexrefs next];
       
       while (activity)
@@ -299,8 +329,8 @@ ensureLeadingMerge (Schedule_c *self, id <Index> index, timeval_t tVal)
           Schedule_c *mergeSchedule;
           Activity_c *swarmActivity;
           
-          scheduleIndex = activity->currentIndex;
-          [scheduleIndex setLoc: Start];
+          scheduleIndex =  (ScheduleIndex_c *)activity->currentIndex;
+          MAP_INDEX_SETLOC (scheduleIndex, Start);
           scheduleIndex->currentAction = 0; 
           scheduleIndex->currentTime = tVal;
           if (scheduleIndex->startTime > tVal)
@@ -308,16 +338,16 @@ ensureLeadingMerge (Schedule_c *self, id <Index> index, timeval_t tVal)
           swarmActivity = activity->swarmActivity;
           if (swarmActivity)
             {
-              mergeScheduleIndex = swarmActivity->currentIndex;
+              mergeScheduleIndex =  (ScheduleIndex_c *)swarmActivity->currentIndex;
               mergeSchedule = 
                 (Schedule_c *) mergeScheduleIndex->collection;
               _activity_insertAction (mergeSchedule,
                                       tVal, 
-                                      activity->mergeAction);
+                                      (CAction *)activity->mergeAction);
             }
           activity = [indexrefs next];
         }
-      [indexrefs drop];
+      DROP (indexrefs);
     }
 }
   
@@ -343,7 +373,7 @@ _activity_insertAction (Schedule_c *self, timeval_t tVal, CAction *anAction)
   
   anAction->owner = (id) self;
   memptr = &anAction;
-  newKey = [self at: (id) tVal memberSlot: &memptr];
+  newKey = MAP_AT_MEMBERSLOT (self, (id) tVal, &memptr);
   
   // if no previous action at key, then return unless singleton group required 
   if (newKey) 
@@ -351,10 +381,10 @@ _activity_insertAction (Schedule_c *self, timeval_t tVal, CAction *anAction)
       if (self->activityRefs
           && _activity_current && getCurrentTime () <= tVal) 
 	{
-          id index = [self createIndex: scratchZone fromMember: anAction];
+          id index = MAP_CREATEINDEX_FROMMEMBER (self, getCZone (getZone (self)), anAction);
           id successor_action;
-          
-          successor_action = [index next];
+
+          successor_action = MAP_INDEX_NEXT (index);
           // Successor action is used for "adjacency test": this
           // action is the successor of action that was just added to
           // the schedule.  If successor action is equal to the action
@@ -373,15 +403,15 @@ _activity_insertAction (Schedule_c *self, timeval_t tVal, CAction *anAction)
               ScheduleActivity_c *activity;
               
               oldTime = (timeval_t) [index getKey];		   
-              indexrefs = [self->activityRefs begin: scratchZone];
-              [indexrefs setLoc: Start];
+              indexrefs = MLIST_BEGIN_ZONE (self->activityRefs, scratchZone);
+              MLIST_INDEX_SETLOC (indexrefs, Start);
               activity = [indexrefs next];
               
               while (activity)
                 {
                   ScheduleIndex_c *scheduleIndex;
                   
-                  scheduleIndex = activity->currentIndex;
+                  scheduleIndex =  (ScheduleIndex_c *)activity->currentIndex;
                   if ([scheduleIndex get] == successor_action) 
                     {
                       // update of index, if we were to perform
@@ -395,10 +425,11 @@ _activity_insertAction (Schedule_c *self, timeval_t tVal, CAction *anAction)
                       // faster way to deal with this would be:
                       // scheduleIndex->position++;
                       
-                      [scheduleIndex setLoc: Start];
+                      MAP_INDEX_SETLOC (scheduleIndex, Start);
                       [scheduleIndex findNext: successor_action];
                       
-                      scheduleIndex->currentAction = [scheduleIndex prev];
+                      scheduleIndex->currentAction =
+			MAP_INDEX_PREV (scheduleIndex);
                       scheduleIndex->currentTime = tVal;
                       _update_mergeSchedules (self,
                                               activity->swarmActivity,
@@ -406,14 +437,14 @@ _activity_insertAction (Schedule_c *self, timeval_t tVal, CAction *anAction)
                     }
                   activity = [indexrefs next];
                 }
-              [indexrefs drop];
+              DROP (indexrefs);
             }
           else
             {
-              [index prev];
+              MAP_INDEX_PREV (index);
               ensureLeadingMerge (self, index, tVal);
             }
-          [index drop];
+          DROP (index);
 	}
       if (!(self->bits & BitSingletonGroups))
 	return;
@@ -428,22 +459,21 @@ _activity_insertAction (Schedule_c *self, timeval_t tVal, CAction *anAction)
 	  existingGroup =
 	    (id) ((ActionConcurrent_c *) existingAction)->concurrentGroup;
 	  anAction->owner = (id) existingGroup;
-	  [existingGroup addLast: anAction];
+	  GENERIC_ADD_LAST (existingGroup, anAction);
 	  return;
 	}
     }
 
   // add initial actions to new group
   newAction = createGroup (self);
-  newAction->ownerActions = existingAction->ownerActions;  // replace mem links
   *memptr = newAction;
   if (!newKey) 
     {
       existingAction->owner = (ActionType_c *) newAction->concurrentGroup;
-      [(id) newAction->concurrentGroup addLast: existingAction];
+      GENERIC_ADD_LAST ((id) newAction->concurrentGroup, existingAction);
     }
   anAction->owner = (id) newAction->concurrentGroup;
-  [(id) newAction->concurrentGroup addLast: anAction];
+  GENERIC_ADD_LAST ((id) newAction->concurrentGroup, anAction);
 }
 
 //
@@ -471,8 +501,7 @@ _activity_insertAction (Schedule_c *self, timeval_t tVal, CAction *anAction)
   // if existing action at key then insert into new group and return group
 
   existingAction->owner = (ActionType_c *) newAction->concurrentGroup;
-  newAction->ownerActions = existingAction->ownerActions;  // replace mem links
-  [(id) newAction->concurrentGroup addLast: existingAction];
+  GENERIC_ADD_LAST ((id) newAction->concurrentGroup, existingAction);
   *memptr = newAction;
   return newAction->concurrentGroup;
 }
@@ -484,24 +513,22 @@ _activity_insertAction (Schedule_c *self, timeval_t tVal, CAction *anAction)
 {
   id  removedAction, emptyAction;
 
+#ifndef FAST
   if (_obj_debug && ![anAction conformsTo: @protocol (Action)])
     raiseEvent (InvalidArgument,
                 "> object to be removed from schedule is not an action\n");
+#endif
 
-  if (_obj_debug && ((CAction *) anAction)->owner == (id) self)
+  if (((CAction *) anAction)->owner == (id) self)
     removedAction = [super remove: anAction];
   else
     {
       // concurrent group
-
-      if (_obj_debug && ![((CAction *) anAction)->owner respondsTo: M(_getEmptyActionConcurrent_)])
-      /*
-      if (_obj_debug && ![((CAction *) anAction)->owner conformsTo: 
-						  @protocol (ConcurrentGroup)]
-	  && ![((CAction *) anAction)->owner conformsTo: 
-						  @protocol (ConcurrentSchedule)])
-      */   raiseEvent (InvalidArgument,
+#ifndef FAST
+      if (_obj_debug && ![((CAction *) anAction)->owner conformsTo: @protocol (ConcurrentGroup)])
+        raiseEvent (InvalidArgument,
                     "> action to be removed from schedule does not belong to schedule\n");
+#endif
       
       removedAction = [(id) ((CAction *) anAction)->owner remove: anAction];
       
@@ -526,7 +553,7 @@ _activity_insertAction (Schedule_c *self, timeval_t tVal, CAction *anAction)
   id <FAction> faction =  [FAction createBegin: getCZone (getZone (self))];
   [faction setCall: call];
   faction = [faction createEnd];
-  _activity_insertAction (self, tVal, faction);
+  _activity_insertAction (self, tVal, (CAction *)faction);
   return faction;
 }
 
@@ -545,7 +572,7 @@ _activity_insertAction (Schedule_c *self, timeval_t tVal, CAction *anAction)
     [ActionCall createBegin: getCZone (getZone (self))];
   [newAction setFunctionPointer: fptr];
   newAction = [newAction createEnd];
-  _activity_insertAction (self, tVal, newAction);
+  _activity_insertAction (self, tVal, (CAction *)newAction);
   return newAction;
 }
 
@@ -556,7 +583,7 @@ _activity_insertAction (Schedule_c *self, timeval_t tVal, CAction *anAction)
   [newAction setFunctionPointer: fptr];
   [newAction setArg1: arg1];
   newAction = [newAction createEnd];
-  _activity_insertAction (self, tVal, newAction);
+  _activity_insertAction (self, tVal, (CAction *)newAction);
   return newAction;
 }
 
@@ -568,7 +595,7 @@ _activity_insertAction (Schedule_c *self, timeval_t tVal, CAction *anAction)
   [newAction setArg1: arg1];
   [newAction setArg2: arg2];
   newAction = [newAction createEnd];
-  _activity_insertAction (self, tVal, newAction);
+  _activity_insertAction (self, tVal, (CAction *)newAction);
   return newAction;
 }
 
@@ -581,7 +608,7 @@ _activity_insertAction (Schedule_c *self, timeval_t tVal, CAction *anAction)
   [newAction setArg2: arg2];
   [newAction setArg3: arg3];
   newAction = [newAction createEnd];
-  _activity_insertAction (self, tVal, newAction);
+  _activity_insertAction (self, tVal, (CAction *)newAction);
   return newAction;
 }
 
@@ -591,7 +618,7 @@ _activity_insertAction (Schedule_c *self, timeval_t tVal, CAction *anAction)
   [newAction setTarget: target];
   [newAction setMessageSelector: aSel];
   newAction = [newAction createEnd];
-  _activity_insertAction (self, tVal, newAction);
+  _activity_insertAction (self, tVal, (CAction *)newAction);
   return newAction;
 }
 
@@ -602,7 +629,7 @@ _activity_insertAction (Schedule_c *self, timeval_t tVal, CAction *anAction)
   [newAction setMessageSelector: aSel];
   [newAction setArg1: arg1];
   newAction = [newAction createEnd];
-  _activity_insertAction (self, tVal, newAction);
+  _activity_insertAction (self, tVal, (CAction *)newAction);
   return newAction;
 }
 
@@ -614,7 +641,7 @@ _activity_insertAction (Schedule_c *self, timeval_t tVal, CAction *anAction)
   [newAction setArg1: arg1];
   [newAction setArg2: arg2];
   newAction = [newAction createEnd];
-  _activity_insertAction (self, tVal, newAction);
+  _activity_insertAction (self, tVal, (CAction *)newAction);
   return newAction;
 }
 
@@ -627,7 +654,19 @@ _activity_insertAction (Schedule_c *self, timeval_t tVal, CAction *anAction)
   [newAction setArg2: arg2];
   [newAction setArg3: arg3];
   newAction = [newAction createEnd];
-  _activity_insertAction (self, tVal, newAction);
+  _activity_insertAction (self, tVal, (CAction *)newAction);
+  return newAction;
+}
+
+- (id <ActionForEachHomogeneous>)at: (timeval_t)tVal createActionForEachHomogeneous: target message: (SEL)aSel
+{
+  id <ActionForEachHomogeneous> newAction =
+    [ActionForEachHomogeneous createBegin:
+                                getCZone (getZone (self))];
+  [newAction setTarget: target];
+  [newAction setMessageSelector: aSel];
+  newAction = [newAction createEnd];
+  _activity_insertAction (self, tVal, (CAction *)newAction);
   return newAction;
 }
 
@@ -638,7 +677,7 @@ _activity_insertAction (Schedule_c *self, timeval_t tVal, CAction *anAction)
   [newAction setTarget: target];
   [newAction setMessageSelector: aSel];
   newAction = [newAction createEnd];
-  _activity_insertAction (self, tVal, newAction);
+  _activity_insertAction (self, tVal, (CAction *)newAction);
   return newAction;
 }
 
@@ -650,7 +689,7 @@ _activity_insertAction (Schedule_c *self, timeval_t tVal, CAction *anAction)
   [newAction setMessageSelector: aSel];
   [newAction setArg1: arg1];
   newAction = [newAction createEnd];
-  _activity_insertAction (self, tVal, newAction);
+  _activity_insertAction (self, tVal, (CAction *)newAction);
   return newAction;
 }
 
@@ -663,7 +702,7 @@ _activity_insertAction (Schedule_c *self, timeval_t tVal, CAction *anAction)
   [newAction setArg1: arg1];
   [newAction setArg2: arg2];
   newAction = [newAction createEnd];
-  _activity_insertAction (self, tVal, newAction);
+  _activity_insertAction (self, tVal, (CAction *)newAction);
   return newAction;
 }
 
@@ -678,7 +717,7 @@ _activity_insertAction (Schedule_c *self, timeval_t tVal, CAction *anAction)
   [newAction setArg2: arg2];
   [newAction setArg3: arg2];
   newAction = [newAction createEnd];
-  _activity_insertAction (self, tVal, newAction);
+  _activity_insertAction (self, tVal, (CAction *)newAction);
   return newAction;
 }
 
@@ -689,7 +728,7 @@ _activity_insertAction (Schedule_c *self, timeval_t tVal, CAction *anAction)
   [newAction setCall: call];
   [newAction setTarget: target];
   newAction = [newAction createEnd];
-  _activity_insertAction (self, tVal, newAction);
+  _activity_insertAction (self, tVal, (CAction *)newAction);
   return newAction;
 }
 
@@ -700,7 +739,7 @@ _activity_insertAction (Schedule_c *self, timeval_t tVal, CAction *anAction)
   [newAction setCall: call];
   [newAction setTarget: target];
   newAction = [newAction createEnd];
-  _activity_insertAction (self, tVal, newAction);
+  _activity_insertAction (self, tVal, (CAction *)newAction);
   return newAction;
 }
 
@@ -758,6 +797,11 @@ _activity_insertAction (Schedule_c *self, timeval_t tVal, CAction *anAction)
   return [self at: 0 createActionTo: target message: aSel : arg1:arg2:arg3];
 }
 
+- (id <ActionForEachHomogeneous>)createActionForEachHomogeneous: target message: (SEL)aSel
+{
+  return [self at: 0 createActionForEachHomogeneous: target message: aSel];
+}
+
 - (id <ActionForEach>)createActionForEach: target message: (SEL)aSel
 {
   return [self at: 0 createActionForEach: target message: aSel];
@@ -814,14 +858,14 @@ _activity_insertAction (Schedule_c *self, timeval_t tVal, CAction *anAction)
               nextMember = [groupIndex next];
               mapObject (mapalloc, groupMember);
             }
-          [groupIndex drop];
+          DROP (groupIndex);
         }
       
       // map the action contained in the schedule itself
       
       mapObject (mapalloc, member);
     }
-  [index drop];
+  DROP (index);
   [super mapAllocations: mapalloc];
 }
 
@@ -844,7 +888,7 @@ _activity_insertAction (Schedule_c *self, timeval_t tVal, CAction *anAction)
       [outputCharStream catC: buffer];
       [actionAtTime describe: outputCharStream];
     }
-  [index drop]; 
+  DROP (index);
 }
 
 - (void)describeForEachID: outputCharStream
@@ -860,7 +904,7 @@ _activity_insertAction (Schedule_c *self, timeval_t tVal, CAction *anAction)
       sprintf (buffer, "at time: %lu action is: ", timeOfAction);
       [actionAtTime describeID: outputCharStream];
     }
-  [index drop]; 
+  DROP (index);
 }
 
 @end
@@ -877,6 +921,11 @@ PHASE(Using)
   [(id) concurrentGroup _performPlan_];
 }
 
+- (id <ActionGroup>)getConcurrentGroup
+{
+  return concurrentGroup;
+}
+
 - (void)mapAllocations: (mapalloc_t)mapalloc
 {
   // identify the action group that held the concurrent actions
@@ -890,9 +939,8 @@ PHASE(Using)
   [outputCharStream catC: "[ActionConcurrent_c "];
   _obj_formatIDString (buffer, self);
   [outputCharStream catC: buffer];
-  [outputCharStream catC: " concurrentGroup:"];
-  _obj_formatIDString (buffer, concurrentGroup);
-  [outputCharStream catC: buffer];
+  [outputCharStream catC: " concurrentGroup:\n"];
+  [concurrentGroup describeForEach: outputCharStream];
   [outputCharStream catC: "]\n"];
 }
 
@@ -981,8 +1029,9 @@ PHASE(Using)
 //
 - (void)addLast: mergeAction
 {
-  [self at: (id) ((ActionMerge_c *) mergeAction)->subactivity->activationNumber
-        insert: mergeAction];
+  MAP_AT_INSERT (self, 
+		 (id) ((ActionMerge_c *) mergeAction)->subactivity->activationNumber,
+		 mergeAction);
 }
 
 //
@@ -1031,7 +1080,7 @@ PHASE(Using)
   id <Symbol> nextStatus = nil;
 
   while ([self getCurrentTime] < tVal
-         && !COMPLETEDP (nextStatus = [self next]));
+         && !COMPLETEDP (nextStatus = [self nextAction]));
   return nextStatus ? nextStatus : [self getStatus];
 }
 
@@ -1110,7 +1159,7 @@ PHASE(Using)
           //!! get actionAtIndex directly from the underlying implementation)
           newAction = [ActionChanged 
                         create: getCZone (getZone ((Activity_c *) activity))];
-          newAction->actionAtIndex = actionAtIndex;
+          newAction->actionAtIndex =  (ActionConcurrent_c *)actionAtIndex;
           currentAction = newAction;
           setMappedAlloc (self);
           // return special object to handle _performAction_:
@@ -1132,7 +1181,7 @@ PHASE(Using)
   // Get next action from index, adjust times, and return.
   //
   
-  currentAction = [self next: (id *) &currentTime];
+  currentAction = MAP_INDEX_NEXTKEY (self, (id *) &currentTime);
   
   if (currentAction)
     { 
@@ -1156,7 +1205,7 @@ PHASE(Using)
             raiseEvent (SourceMessage,
                         "> schedule did not complete soon enough for its scheduled repeat\n");
           
-          [self setLoc: Start];
+          MAP_INDEX_SETLOC (self, Start);
           currentAction = [self next: (id *) &currentTime];
           currentTime += startTime;
         }
@@ -1174,15 +1223,15 @@ PHASE(Using)
       && (currentAction
           || INITIALIZEDP (activity->swarmActivity->status)))
     {
-      swarmIndex = activity->swarmActivity->currentIndex;
+      swarmIndex = (ScheduleIndex_c *)activity->swarmActivity->currentIndex;
 
       _activity_insertAction ((Schedule_c *) swarmIndex->collection,
                               currentTime,
-                              activity->mergeAction);
+			      (CAction *)activity->mergeAction);
       if (currentAction) 
         *status = Holding;
       else
-        [self setLoc: Start];
+        MAP_INDEX_SETLOC (self, Start);
       
       // if empty schedule just added to new swarm activity, then set to
       // reprocess schedule (should generalize to any empty schedule)
@@ -1211,7 +1260,7 @@ PHASE(Using)
 
 - setActivity: (id <Activity>)theActivity
 {
-  activity = theActivity;
+  activity = (ScheduleActivity_c *)theActivity;
   return self;
 }
 
@@ -1239,14 +1288,26 @@ PHASE(Using)
     }
   else
     {
+#ifdef FAST
+      Class class = getClass (collection);
+#endif
       // just remove the action at the index
       removedAction = [super remove];
-      
+
       // Avoid leaving behind invalid owners in mergeAction
       // `owner' is used in [Schedule remove:], for example
+      // first do some common cases before expensive conformsTo
+#ifdef FAST
+      if (class != id_Schedule_c)
+        if (class == id_ActivationOrder_c || class == id_ConcurrentSchedule_c
+            || [collection conformsTo: @protocol (ConcurrentSchedule)])
+#else
       if ([collection conformsTo: @protocol (ConcurrentSchedule)])
-        removedAction->owner =
-          ((ConcurrentSchedule_c *) collection)->actionConcurrent->owner;
+#endif
+{
+          removedAction->owner =
+            ((ConcurrentSchedule_c *) collection)->actionConcurrent->owner;
+}
     }
   currentAction = (id <Action>) Removed;
   return removedAction;
@@ -1289,7 +1350,7 @@ PHASE(Using)
 //
 - (void)dropAllocations: (BOOL)componentAlloc
 {
-  [((Schedule_c *) collection)->activityRefs remove: activity];
+  ORDEREDSET_REMOVE (((Schedule_c *) collection)->activityRefs, activity);
   [super dropAllocations: YES];
 }
 
@@ -1326,7 +1387,7 @@ PHASE(Using)
   ScheduleIndex_c *schedIndex;
   id unusedStatus;
 
-  schedIndex = ((Activity_c *) anActivity)->currentIndex;
+  schedIndex = (ScheduleIndex_c *)((Activity_c *) anActivity)->currentIndex;
   schedIndex->currentAction = actionAtIndex;
   unsetMappedAlloc (schedIndex);
   [actionAtIndex _performAction_: anActivity];
